/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: ooxcontexthandler.hxx,v $
 *
 *  $Revision: 1.1.2.7 $
 *
 *  last change: $Author: dr $ $Date: 2007/12/06 11:48:36 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef OOX_XLS_OOXCONTEXTHANDLER_HXX
#define OOX_XLS_OOXCONTEXTHANDLER_HXX

#include "oox/core/context.hxx"
#include "oox/xls/ooxfragmenthandler.hxx"

namespace oox {
namespace xls {

// ============================================================================

class OoxContextHandler : public ::oox::core::Context, public OoxContextHelper
{
protected:
    explicit            OoxContextHandler( const OoxFragmentHandler& rParent );

    // com.sun.star.xml.sax.XFastContextHandler interface ---------------------

    virtual ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastContextHandler > SAL_CALL
                        createFastChildContext(
                            sal_Int32 nElement,
                            const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastAttributeList >& rxAttribs )
                            throw(  ::com::sun::star::xml::sax::SAXException,
                                    ::com::sun::star::uno::RuntimeException );

    virtual void SAL_CALL startFastElement(
                            sal_Int32 nElement,
                            const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastAttributeList >& rxAttribs )
                            throw(  ::com::sun::star::xml::sax::SAXException,
                                    ::com::sun::star::uno::RuntimeException );

    virtual void SAL_CALL characters( const ::rtl::OUString& rChars )
                            throw(  ::com::sun::star::xml::sax::SAXException,
                                    ::com::sun::star::uno::RuntimeException );

    virtual void SAL_CALL endFastElement( sal_Int32 nElement )
                            throw(  ::com::sun::star::xml::sax::SAXException,
                                    ::com::sun::star::uno::RuntimeException );

    // oox.core.RecordContext interface ---------------------------------------

    virtual ::oox::core::RecordContextRef createRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm );

    virtual void        startRecord( sal_Int32 nRecId, RecordInputStream& rStrm );

    virtual void        endRecord( sal_Int32 nRecId );

    // oox.xls.OoxContextHelper interface -------------------------------------

    virtual bool        onCanCreateContext( sal_Int32 nElement ) const;
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastContextHandler >
                        onCreateContext( sal_Int32 nElement, const AttributeList& rAttribs );
    virtual void        onStartElement( const AttributeList& rAttribs );
    virtual void        onEndElement( const ::rtl::OUString& rChars );

    virtual bool        onCanCreateRecordContext( sal_Int32 nRecId );
    virtual ::oox::core::RecordContextRef
                        onCreateRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm );
    virtual void        onStartRecord( RecordInputStream& rStrm );
    virtual void        onEndRecord();
};

// ============================================================================

template< typename HelperType >
class OoxHelperContextBase : public OoxContextHandler, public HelperType
{
public:
    template< typename ParentType >
    inline explicit     OoxHelperContextBase( const ParentType& rParent, const HelperType& rHelper ) :
                            OoxContextHandler( rParent ), HelperType( rHelper ) {}

    template< typename ParentType >
    inline explicit     OoxHelperContextBase( const ParentType& rParent ) :
                            OoxContextHandler( rParent ), HelperType( rParent ) {}
};

// ============================================================================

class OoxWorkbookContextBase : public OoxContextHandler, public WorkbookHelper
{
public:
    template< typename ParentType >
    explicit            OoxWorkbookContextBase( const ParentType& rParent );
};

// ----------------------------------------------------------------------------

template< typename ParentType >
OoxWorkbookContextBase::OoxWorkbookContextBase( const ParentType& rParent ) :
    OoxContextHandler( rParent ),
    WorkbookHelper( rParent )
{
}

// ============================================================================

class OoxWorksheetContextBase : public OoxContextHandler, public WorksheetHelperRoot
{
public:
    template< typename ParentType >
    explicit            OoxWorksheetContextBase(
                            const ParentType& rParent,
                            ISegmentProgressBarRef xProgressBar,
                            WorksheetType eSheetType,
                            sal_Int32 nSheet );

    template< typename ParentType >
    explicit            OoxWorksheetContextBase( const ParentType& rParent );
};

// ----------------------------------------------------------------------------

template< typename ParentType >
OoxWorksheetContextBase::OoxWorksheetContextBase( const ParentType& rParent,
        ISegmentProgressBarRef xProgressBar, WorksheetType eSheetType, sal_Int32 nSheet ) :
    OoxContextHandler( rParent ),
    WorksheetHelperRoot( rParent, xProgressBar, eSheetType, nSheet )
{
}

template< typename ParentType >
OoxWorksheetContextBase::OoxWorksheetContextBase( const ParentType& rParent ) :
    OoxContextHandler( rParent ),
    WorksheetHelperRoot( rParent )
{
}

// ============================================================================

} // namespace xls
} // namespace oox

#endif

