/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: ooxcontexthandler.cxx,v $
 *
 *  $Revision: 1.1.2.5 $
 *
 *  last change: $Author: dr $ $Date: 2007/12/05 14:28:24 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#include "oox/xls/ooxcontexthandler.hxx"

using ::rtl::OUString;
using ::com::sun::star::uno::Reference;
using ::com::sun::star::uno::RuntimeException;
using ::com::sun::star::xml::sax::SAXException;
using ::com::sun::star::xml::sax::XFastContextHandler;
using ::com::sun::star::xml::sax::XFastAttributeList;
using ::oox::core::Context;
using ::oox::core::FragmentHandlerRef;
using ::oox::core::RecordContextRef;

namespace oox {
namespace xls {

// ============================================================================

OoxContextHandler::OoxContextHandler( const OoxFragmentHandler& rParent ) :
    Context( const_cast< OoxFragmentHandler* >( &rParent ) ),
    OoxContextHelper( rParent )
{
}

// com.sun.star.xml.sax.XFastContextHandler interface -------------------------

Reference< XFastContextHandler > SAL_CALL OoxContextHandler::createFastChildContext(
        sal_Int32 nElement, const Reference< XFastAttributeList >& rxAttribs ) throw( SAXException, RuntimeException )
{
    return implCreateChildContext( nElement, rxAttribs );
}

void SAL_CALL OoxContextHandler::startFastElement(
        sal_Int32 nElement, const Reference< XFastAttributeList >& rxAttribs ) throw( SAXException, RuntimeException )
{
    implStartCurrentContext( nElement, rxAttribs );
}

void SAL_CALL OoxContextHandler::characters( const OUString& rChars ) throw( SAXException, RuntimeException )
{
    implCharacters( rChars );
}

void SAL_CALL OoxContextHandler::endFastElement( sal_Int32 nElement ) throw( SAXException, RuntimeException )
{
    implEndCurrentContext( nElement );
}

// oox.core.RecordContext interface -------------------------------------------

RecordContextRef OoxContextHandler::createRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm )
{
    return implCreateRecordContext( nRecId, rStrm );
}

void OoxContextHandler::startRecord( sal_Int32 nRecId, RecordInputStream& rStrm )
{
    implStartRecord( nRecId, rStrm );
}

void OoxContextHandler::endRecord( sal_Int32 nRecId )
{
    implEndRecord( nRecId );
}

// oox.xls.OoxContextHelper interface -----------------------------------------

bool OoxContextHandler::onCanCreateContext( sal_Int32 ) const
{
    return false;
}

Reference< XFastContextHandler > OoxContextHandler::onCreateContext( sal_Int32, const AttributeList& )
{
    // default behaviour: return this to reuse the same instance
    return this;
}

void OoxContextHandler::onStartElement( const AttributeList& )
{
}

void OoxContextHandler::onEndElement( const OUString& )
{
}

bool OoxContextHandler::onCanCreateRecordContext( sal_Int32 )
{
    return false;
}

RecordContextRef OoxContextHandler::onCreateRecordContext( sal_Int32, RecordInputStream& )
{
    // default behaviour: return this to reuse the same instance
    return this;
}

void OoxContextHandler::onStartRecord( RecordInputStream& )
{
}

void OoxContextHandler::onEndRecord()
{
}

// ============================================================================

} // namespace xls
} // namespace oox

