/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: ooxfragmenthandler.cxx,v $
 *
 *  $Revision: 1.1.2.9 $
 *
 *  last change: $Author: dr $ $Date: 2007/12/06 11:49:06 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#include "oox/xls/ooxfragmenthandler.hxx"

using ::rtl::OUString;
using ::com::sun::star::uno::Reference;
using ::com::sun::star::uno::RuntimeException;
using ::com::sun::star::io::IOException;
using ::com::sun::star::io::XInputStream;
using ::com::sun::star::xml::sax::SAXException;
using ::com::sun::star::xml::sax::XFastContextHandler;
using ::com::sun::star::xml::sax::XFastAttributeList;
using ::oox::core::FragmentHandler;
using ::oox::core::RecordContextRef;
using ::oox::core::Relation;
using ::oox::core::Relations;
using ::oox::core::RelationsRef;
using ::oox::core::XmlFilterBase;

namespace oox {
namespace xls {

// ============================================================================

OoxFragmentHandler::OoxFragmentHandler( XmlFilterBase& rFilter, const OUString& rFragmentPath ) :
    FragmentHandler( &rFilter, rFragmentPath )
{
}

// com.sun.star.xml.sax.XFastDocumentHandler interface --------------------

void SAL_CALL OoxFragmentHandler::startDocument() throw( SAXException, RuntimeException )
{
    initializeImport();
}

void SAL_CALL OoxFragmentHandler::endDocument() throw( SAXException, RuntimeException )
{
    finalizeImport();
}

// com.sun.star.xml.sax.XFastContextHandler interface -------------------------

Reference< XFastContextHandler > SAL_CALL OoxFragmentHandler::createFastChildContext(
        sal_Int32 nElement, const Reference< XFastAttributeList >& rxAttribs ) throw( SAXException, RuntimeException )
{
    return implCreateChildContext( nElement, rxAttribs );
}

void SAL_CALL OoxFragmentHandler::startFastElement(
        sal_Int32 nElement, const Reference< XFastAttributeList >& rxAttribs ) throw( SAXException, RuntimeException )
{
    implStartCurrentContext( nElement, rxAttribs );
}

void SAL_CALL OoxFragmentHandler::characters( const OUString& rChars ) throw( SAXException, RuntimeException )
{
    implCharacters( rChars );
}

void SAL_CALL OoxFragmentHandler::endFastElement( sal_Int32 nElement ) throw( SAXException, RuntimeException )
{
    implEndCurrentContext( nElement );
}

// oox.core.RecordContext interface -------------------------------------------

RecordContextRef OoxFragmentHandler::createRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm )
{
    return implCreateRecordContext( nRecId, rStrm );
}

void OoxFragmentHandler::startRecord( sal_Int32 nRecId, RecordInputStream& rStrm )
{
    implStartRecord( nRecId, rStrm );
}

void OoxFragmentHandler::endRecord( sal_Int32 nRecId )
{
    implEndRecord( nRecId );
}

// oox.xls.OoxContextHelper interface -----------------------------------------

bool OoxFragmentHandler::onCanCreateContext( sal_Int32 ) const
{
    return false;
}

Reference< XFastContextHandler > OoxFragmentHandler::onCreateContext( sal_Int32, const AttributeList& )
{
    // default behaviour: return this to reuse the same instance
    return this;
}

void OoxFragmentHandler::onStartElement( const AttributeList& )
{
}

void OoxFragmentHandler::onEndElement( const OUString& )
{
}

bool OoxFragmentHandler::onCanCreateRecordContext( sal_Int32 )
{
    return false;
}

RecordContextRef OoxFragmentHandler::onCreateRecordContext( sal_Int32, RecordInputStream& )
{
    // default behaviour: return this to reuse the same instance
    return this;
}

void OoxFragmentHandler::onStartRecord( RecordInputStream& )
{
}

void OoxFragmentHandler::onEndRecord()
{
}

// oox.xls.OoxFragmentHandler interface ---------------------------------------

void OoxFragmentHandler::initializeImport()
{
}

void OoxFragmentHandler::finalizeImport()
{
}

// ============================================================================

OoxWorkbookFragmentBase::OoxWorkbookFragmentBase(
        const WorkbookHelper& rHelper, const OUString& rFragmentPath ) :
    OoxFragmentHandler( rHelper.getOoxFilter(), rFragmentPath ),
    WorkbookHelper( rHelper )
{
}

// ============================================================================

OoxWorksheetFragmentBase::OoxWorksheetFragmentBase( const WorkbookHelper& rHelper,
        const OUString& rFragmentPath, ISegmentProgressBarRef xProgressBar, WorksheetType eSheetType, sal_Int32 nSheet ) :
    OoxFragmentHandler( rHelper.getOoxFilter(), rFragmentPath ),
    WorksheetHelperRoot( rHelper, xProgressBar, eSheetType, nSheet )
{
}

OoxWorksheetFragmentBase::OoxWorksheetFragmentBase(
        const WorksheetHelper& rHelper, const OUString& rFragmentPath ) :
    OoxFragmentHandler( rHelper.getOoxFilter(), rFragmentPath ),
    WorksheetHelperRoot( rHelper )
{
}

// ============================================================================

} // namespace xls
} // namespace oox

